/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.web.builders.ResourcePageBuilder;
import org.wso2.registry.web.custom.CustomUIManager;
import org.wso2.registry.web.custom.CustomUIInitializer;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.net.URLDecoder;

/**
 * This servlet is responsible for building the Resources page of the UI.
 */
public class WebServlet extends HttpServlet {

    private static final Log log = LogFactory.getLog(WebServlet.class);

    private String contextRoot;

    private int serverPrefixLength;

    private CustomUIManager customUIManager;

    public void init(ServletConfig servletConfig) throws ServletException {
        super.init(servletConfig);

        customUIManager = new CustomUIManager();

        String configPath = servletConfig.getInitParameter(RegistryConstants.REGISTRY_CONFIG_PATH);
        InputStream configStream;
        if (configPath != null) {
            try {
                configStream = new FileInputStream(configPath);
            } catch (FileNotFoundException e) {
                throw new ServletException("Couldn't find specified config file '" +
                        configPath + "'", e);
            }
        } else {
            configStream =
                    servletConfig.getServletContext().getResourceAsStream("/WEB-INF/registry.xml");
        }

        try {
            CustomUIInitializer.initialize(configStream, servletConfig, customUIManager);
        } catch (RegistryException e) {
            String msg = "Failed to initialize custom UIs. Caused by: " + e.getMessage();
            log.error(msg, e);
        }

        servletConfig.getServletContext().
                setAttribute(UIConstants.CUSTOM_UI_MANAGER, customUIManager);
    }

    /**
     * To find the name of the war , so getting that from the request context path
     *
     * @param req :  HttpServletRequest
     */
    public void initContextRoot(HttpServletRequest req) {
        if (contextRoot != null && contextRoot.trim().length() != 0) {
            return;
        }
        String contextPath = req.getContextPath();

        //handling ROOT scenario, for servlets in the default (root) context, this method returns ""
        if (contextPath != null && contextPath.length() == 0) {
            contextPath = "/";
        }
        this.contextRoot = contextPath;

        serverPrefixLength = contextRoot.length() +
                RegistryConstants.PATH_SEPARATOR.length() + UIConstants.WEB_PATH.length();
    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        initContextRoot(request);

        String uri = URLDecoder.decode(request.getRequestURI(), "UTF-8");
        String path = uri.substring(serverPrefixLength, uri.length());

        ResourcePath resourcePath = new ResourcePath(path);

        request.getSession().setAttribute(UIConstants.PROCESSED_PATH, resourcePath);

        String viewKey = resourcePath.getParameterValue(UIConstants.VIEW_KEY);
        try {
            customUIManager.populateResourceView(
                    resourcePath.getPathWithVersion(), viewKey, request);
        } catch (RegistryException e) {
            String msg = "Failed to generate custom UIs for the resource " +
                    resourcePath.getCompletePath() + ". Resource UI will be displayed without " +
                    "customizations. Caused by: " + e.getMessage();
            log.error(msg, e);
        }

        ResourcePageBuilder.build(resourcePath, request, response);
    }
}
