/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.custom;

import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axiom.om.OMElement;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.exceptions.RegistryException;

import javax.xml.stream.XMLStreamException;
import javax.xml.namespace.QName;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import java.io.*;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

/**
 * Custom UI configuration format:
 *
 * <customUI>
 *
 *     <mediaType>...</mediaType>
 *     <class>...</class>
 *
 *     <jsp>...</jsp>
 *     <jsp>...</jsp>
 *         ...
 *
 * </customUI>
 */
public class CustomUIInitializer {

    private static final Log log = LogFactory.getLog(CustomUIInitializer.class);

    public static void initialize(
            InputStream configStream, ServletConfig config, CustomUIManager customUIManager)
            throws RegistryException {

        // read the config file
        // extract all listed JSPs to appropraite locations
        // initialize CustomUIManager with mediaType -> class instance map

        try {
            StAXOMBuilder buillder = new StAXOMBuilder(configStream);
            OMElement configElement = buillder.getDocumentElement();
            if (configElement != null) {

                Iterator customUIs = configElement.getChildrenWithName(new QName("customUI"));
                while (customUIs.hasNext()) {

                    String mediaType;
                    String className;
                    List <String> jspPaths = new ArrayList <String> ();

                    OMElement customUIElement = (OMElement) customUIs.next();
                    OMElement mediaTypeElement =
                            customUIElement.getFirstChildWithName(new QName("mediaType"));
                    if (mediaTypeElement != null) {
                        mediaType = mediaTypeElement.getText();
                    } else {
                        String msg = "Media type of the custom UI processor is not specified. " +
                                "Skipping the initialization of custom UI processor.";
                        log.error(msg);
                        continue;
                    }

                    OMElement classElement =
                            customUIElement.getFirstChildWithName(new QName("class"));
                    if (classElement != null) {
                        className = classElement.getText();
                    } else {
                        String msg = "Class name of the custom UI processor is not specified. " +
                                "Skipping the initialization of custom UI processor for " +
                                mediaType + " media type.";
                        log.error(msg);
                        continue;
                    }

                    try {
                        Class processorClass = Class.forName(className);
                        CustomUIProcessor customUIProcessor =
                                (CustomUIProcessor) processorClass.newInstance();
                        customUIManager.registerProcessor(mediaType, customUIProcessor);

                    } catch (ClassNotFoundException e) {
                        String msg = "Custom UI processor class " + className +
                                " could not be found. Skipping the initialization. " +
                                "Caused by: " + e.getMessage();
                        log.error(msg, e);
                        continue;

                    } catch (Exception ge) {

                        String msg = "Failed to instantiate custom UI processor class " +
                                className + ". Skipping the initialization. Caused by: " +
                                ge.getMessage();
                        log.error(msg, ge);
                        continue;

                    }

                    Iterator jsps = customUIElement.getChildrenWithName(new QName("resource"));
                    while (jsps.hasNext()) {
                        OMElement jspElement = (OMElement) jsps.next();
                        jspPaths.add(jspElement.getText());
                    }

                    try {
                        extractJSPs(jspPaths, config);
                    } catch (RegistryException e) {
                        String msg = "Failed to extract JSPs used by custom UI processor of " +
                                "media type: " + mediaType + ". Custom UI processot will be " +
                                "unregistered. Caused by: " + e.getMessage();
                        log.error(msg, e);
                        customUIManager.unregisterProcessor(mediaType);
                        continue;
                    }
                }

            }

        } catch (XMLStreamException e) {

            String msg = "Failed to read the registry configuration file. " +
                    "Make sure that the configuration file is in proper XML format. " +
                    "Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            if (configStream != null) {
                try {
                    configStream.close();
                } catch (IOException e) {
                    String msg = "Failed to close input stream of the registry " +
                            "configuration file. " + e.getMessage();
                    log.error(msg, e);
                    throw new RegistryException(msg, e);
                }
            }
        }
    }

    private static void extractJSPs(List <String> jspPaths, ServletConfig config)
            throws RegistryException {

        String jspHome = config.getServletContext().getRealPath("admin");

        Iterator <String> jsps = jspPaths.iterator();
        while (jsps.hasNext()) {

            String jspPath = jsps.next();

            File jspFile = new File(jspHome + "/" + jspPath);
            if (!jspFile.exists()) {
                try {
                    jspFile.createNewFile();
                } catch (IOException e) {
                    String msg = "Failed to extract the JSP file: " + jspPath +
                            " used to generate custom UIs. " + e.getMessage();
                    log.error(msg, e);
                    throw new RegistryException(msg, e);
                }
            }

            InputStream jspStream = null;
            OutputStream out = null;
            try {
                jspStream = Thread.currentThread().
                    getContextClassLoader().getResourceAsStream(jspPath);

                out = new BufferedOutputStream(new FileOutputStream(jspFile));
                byte[] contentChunk = new byte[1024];
                int byteCount;
                while ((byteCount = jspStream.read(contentChunk)) != -1) {
                    out.write(contentChunk, 0, byteCount);
                }
                out.flush();
                
            } catch (IOException e) {

                String msg = "Failed to copy archived JSP content to the extracted " +
                        "file used to generate custom UIs. " + e.getMessage();
                log.error(msg, e);
                throw new RegistryException(msg, e);

            } finally {

                if (jspStream != null) {
                    try {
                        jspStream.close();
                    } catch (IOException e) {
                        log.error(e);
                    }
                }

                if (out != null) {
                    try {
                        out.close();
                    } catch (IOException e) {
                        log.error(e);
                    }
                }
            }


        }
    }
}
