/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.custom;

import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.processors.ResourceCreationUIProcessor;
import org.wso2.registry.web.builders.ResourcePageBuilder;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.Registry;
import org.wso2.registry.Resource;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.exceptions.RegistryException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;
import java.util.HashMap;

public class CustomUIManager {

    private static final Log log = LogFactory.getLog(CustomUIManager.class);

    Map <String, CustomUIProcessor> processors = new HashMap <String, CustomUIProcessor> ();

    /**
     * todo: This is used for backward compatibility with old custom UI plugins.
     */
    private ResourceCreationUIProcessor resourceCreationUIProcessor;

    public CustomUIManager() {
        // todo: This is used for backward compatibility with old custom UI plugins.
        resourceCreationUIProcessor = new ResourceCreationUIProcessor();
    }

    public void registerProcessor(String mediaType, CustomUIProcessor processor) {
        processors.put(mediaType, processor);
    }

    public void unregisterProcessor(String mediaType) {
        processors.remove(mediaType);
    }

    public void populateResourceView(String path, String viewKey, HttpServletRequest request)
            throws RegistryException {

        if (UIConstants.RAW_VIEW.equals(viewKey)) {
            return;
        }

        String mediaType = getMediaType(path, request);
        CustomUIProcessor processor = processors.get(mediaType);
        if (processor != null) {
            processor.setRequest(request);
            viewKey = (viewKey == null || "".equals(viewKey)) ?
                    processor.getDefaultResourceView() : viewKey;
            CustomUIBean bean = processor.processResourceView(path, viewKey, request);

            bean.setResponseMethod(CustomUIBean.EMBED);
            bean.setResourceViews(processor.getResourceViews());
            request.setAttribute(UIConstants.CUSTOM_UI_BEAN, bean);
        }
    }

    public void generateCreateHome(HttpServletRequest request, HttpServletResponse response)
            throws RegistryException {

        String mediaType = request.getParameter(UIConstants.MEDIA_TYPE);
        String parentPath = request.getParameter(UIConstants.PARENT_PATH);

        CustomUIProcessor processor = processors.get(mediaType);
        if (processor == null) {

            // todo: This is used for backward compatibility with old custom UI plugins.
            resourceCreationUIProcessor.process(request, response);
            return;

            // todo: uncomment once old custom UIs are removed
            //String msg = "Custom UI for creating resources is not available for the media type: " +
            //        mediaType;
            //log.error(msg);
            //CommonUtil.sendErrorContent(response, msg);
            //return;
        }

        Map <String, String> createViews = processor.getCreateViews();
        if (createViews.size() == 1) {
            generateCreateView(request, response);
            return;
        }

        CreateOptionsBean createOptionsBean = new CreateOptionsBean();
        createOptionsBean.setMediaType(mediaType);
        createOptionsBean.setParentPath(parentPath);
        createOptionsBean.setCreateViews(createViews);

        request.setAttribute(UIConstants.CREATE_OPTIONS_BEAN, createOptionsBean);
        CommonUtil.forwardToPage(request, response, UIConstants.CREATE_OPTIONS_JSP);
    }

    public void generateCreateView(HttpServletRequest request, HttpServletResponse response)
            throws RegistryException {

        String mediaType = request.getParameter(UIConstants.MEDIA_TYPE);
        String viewKey = request.getParameter(UIConstants.VIEW_KEY);

        CustomUIProcessor processor = processors.get(mediaType);

        if (processor == null) {

            // todo: This is used for backward compatibility with old custom UI plugins.
            resourceCreationUIProcessor.process(request, response);
            return;

            // todo: uncomment once old custom UIs are removed
            //String msg = "Custom UI for creating resources is not available for the media type: " +
            //        mediaType;
            //log.error(msg);
            //CommonUtil.sendErrorContent(response, msg);
            //return;
        }

        processor.setRequest(request);
        viewKey = (viewKey == null || "".equals(viewKey)) ?
                    processor.getDefaultCreateView() : viewKey;
        CustomUIBean bean = processor.processCreateView(request, viewKey);
        if (bean.getJspPath() != null) {
            request.setAttribute(UIConstants.CUSTOM_UI_BEAN, bean);
            CommonUtil.forwardToPage(request, response, bean.getJspPath());

        } else {
            CommonUtil.sendContent(response, bean.getContentString());
        }
    }

    public void process(HttpServletRequest request, HttpServletResponse response)
            throws RegistryException {

        String mediaType = request.getParameter(UIConstants.MEDIA_TYPE);
        CustomUIProcessor processor = processors.get(mediaType);
        if (processor == null) {
            String msg = "Customized UI processing is not supported for media type: " +
                    mediaType + ". Please make sure that custom UI processors are " +
                    "correctly installed.";
            log.error(msg);
            CommonUtil.sendErrorContent(response, msg);
            return;
        }

        String command = request.getParameter(UIConstants.COMMAND);
        processor.setRequest(request);
        CustomUIBean cuib = processor.process(command, request);

        // now we need to decide how to send the response by examining the cuib
        sendResponse(cuib, mediaType, command, request, response);
    }

    private void sendResponse(CustomUIBean cuib, String mediaType, String command,
                              HttpServletRequest request, HttpServletResponse response) {

        if (CustomUIBean.REDIRECT == cuib.getResponseMethod()) {
            if (cuib.getResourcePath() == null) {
                String msg = "Redirect path is not specified by the custom UI processor " +
                        "of media tyep: " + mediaType;
                log.error(msg);
                CommonUtil.sendErrorContent(response, msg);
                return;
            }

            String url = "/wso2registry/web" + cuib.getResourcePath();
            CommonUtil.redirect(response, url);

        } else if (CustomUIBean.CONTENT_ONLY == cuib.getResponseMethod()) {

            if (cuib.getContentString() != null) {
                CommonUtil.sendContent(response, cuib.getContentString());

            } else if (cuib.getJspPath() != null) {
                request.setAttribute(UIConstants.CUSTOM_UI_BEAN, cuib);
                CommonUtil.forwardToPage(request, response, cuib.getJspPath());

            } else {
                String msg = "Content or a JSP to generate content is not set by the " +
                        "custom UI processor of media type: " + mediaType +
                        " for the command: " + command;
                log.error(msg);
                CommonUtil.sendErrorContent(response, msg);
            }

        } else if (CustomUIBean.EMBED == cuib.getResponseMethod()) {

            String path = cuib.getResourcePath();
            if (path == null) {
                path = request.getParameter(UIConstants.RESOURCE_PATH);
                if (path == null) {
                    String msg = "Resource path is not specified for embedding the custom UI " +
                            "content for media type: " + mediaType + ". Resource path has to be " +
                            "either specified by the custom UI processor or as a " +
                            "request parameter.";
                    log.error(msg);
                    CommonUtil.sendErrorContent(response, msg);
                    return;
                }
            }
            ResourcePath resourcePath = new ResourcePath(path);
            request.setAttribute(UIConstants.CUSTOM_UI_BEAN, cuib);

            ResourcePageBuilder.build(resourcePath, request, response);
        }
    }

    private String getMediaType(String path, HttpServletRequest request) throws RegistryException {

        Registry registry = CommonUtil.getUserRegistry(request);
        Resource resource = registry.get(path);
        return resource.getMediaType();
    }
}
